<?php

namespace Unific\Connector\Helper;

use Magento\Framework\Exception\InputException;
use Symfony\Component\Config\Definition\Exception\Exception;

class Queue extends \Magento\Framework\App\Helper\AbstractHelper
{
    protected $logger;
    protected $restConnection;
    protected $queueCollectionFactory;

    protected $lockCollectionFactory;
    protected $lockFactory;

    protected $hmacHelper;
    protected $productMetadata;
    protected $moduleList;

    protected $fileLog;

    /**
     * OrderPlugin constructor.
     * @param \Magento\Framework\App\Helper\Context $context
     * @param Logger $logger
     * @param Hmac $hmacHelper
     * @param \Unific\Connector\Model\ResourceModel\Queue\CollectionFactory $queueCollectionFactory
     * @param \Unific\Connector\Model\ResourceModel\Lock\CollectionFactory $lockCollectionFactory
     * @param \Unific\Connector\Model\LockFactory $lockFactory
     * @param \Unific\Connector\Connection\Connection $restConnection
     * @param \Magento\Framework\App\ProductMetadataInterface $productMetadata
     * @param \Magento\Framework\Module\ModuleListInterface $moduleList
     */
    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        Logger $logger,
        Hmac $hmacHelper,
        \Unific\Connector\Model\ResourceModel\Queue\CollectionFactory $queueCollectionFactory,
        \Unific\Connector\Model\ResourceModel\Lock\CollectionFactory $lockCollectionFactory,
        \Unific\Connector\Model\LockFactory $lockFactory,
        \Unific\Connector\Connection\Connection $restConnection,
        \Magento\Framework\App\ProductMetadataInterface $productMetadata,
        \Magento\Framework\Module\ModuleListInterface $moduleList,
        \Psr\Log\LoggerInterface $fileLog
    ) {
        parent::__construct($context);

        $this->logger = $logger;
        $this->restConnection = $restConnection;
        $this->queueCollectionFactory = $queueCollectionFactory;

        $this->lockCollectionFactory = $lockCollectionFactory;
        $this->lockFactory = $lockFactory;

        $this->hmacHelper = $hmacHelper;
        $this->productMetadata = $productMetadata;
        $this->moduleList = $moduleList;

        $this->fileLog = $fileLog;
    }

    /**
     * @return void
     */
    public function process()
    {
        if($this->scopeConfig->getValue('unific/connector/integration', \Magento\Store\Model\ScopeInterface::SCOPE_STORE) == "") {
            $this->fileLog->info('Could not process the Unific queue because the integration id was not set. Please configure the extension');
            return;
        }

        $collection = $this->lockCollectionFactory->create();
        $collection->addFieldToFilter('type', 'queue');

        if($collection->getSize() == 0) {
            $this->lockQueue();

            $this->sendDataFromQueue(false, Settings::QUEUE_LIVE_PER_MINUTE);
            $this->sendDataFromQueue(true, Settings::QUEUE_HISTORICAL_PER_MINUTE);

            $this->unlockQueue();
        }
    }

    /**
     * Lock the queue
     */
    protected function lockQueue()
    {
        $lock = $this->lockFactory->create();
        $lock->setType('queue');
        $lock->save();
    }

    /**
     * Unlock the queue
     */
    protected function unlockQueue()
    {
        $collection = $this->lockCollectionFactory->create();
        $collection->addFieldToFilter('type', 'queue');
        $collection->walk('delete');
    }

    /**
     * @param bool $isHistorical
     * @param int $size
     */
    protected function sendDataFromQueue($isHistorical = false, $size = 100)
    {
        $collection = $this->queueCollectionFactory->create();
        $collection->addFieldToFilter('historical', ['eq' => (int)$isHistorical]);
        $collection->addFieldToFilter('retry_amount', ['lt' => Settings::QUEUE_MAX_RETRIES]);
        $collection->setPageSize($size);

        // Sort the queue by retries then priority
        $collection->getSelect()->order(array('retry_amount ASC', 'priority ASC'));

        $collection->setCurPage(1);

        // For efficiency we do not loop everything constantly
        $headers = array();
        $headers['Content-Type'] = 'application/json';
        $headers['Accept'] = 'application/json';
        $headers['X-UNIFIC-INTEGRATION-ID'] = $this->scopeConfig->getValue('unific/connector/integration', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        $headers['X-MAGENTO-VERSION'] = $this->productMetadata->getVersion();
        $headers['X-UNIFIC-CONNECTOR-VERSION'] = $this->moduleList->getOne('Unific_Connector')['setup_version'];

        foreach ($collection as $queueItem) {
            $queueData = json_decode($queueItem->getMessage(), true);

            if(!is_array($queueData)) {
                // Increment the retry amount
                $queueItem->setRetryAmount($queueItem->getRetryAmount() + 1);
                $queueItem->save();

                $this->logger->createLogForCanceledWebhook($queueItem, $queueItem->getGuid() . 'Cannot send message, body empty');
                continue;
            }

            if ($queueItem->getUrl() != '' && filter_var($queueItem->getUrl(), FILTER_VALIDATE_URL) !== false) {
                switch ($queueItem->getRequestType()) {
                    case 'POST':
                        $type = \Zend\Http\Request::METHOD_POST;
                        break;
                    case 'PUT':
                        $type = \Zend\Http\Request::METHOD_PUT;
                        break;
                    case 'DELETE':
                        $type = \Zend\Http\Request::METHOD_DELETE;
                        break;
                    default:
                        $type = \Zend\Http\Request::METHOD_GET;
                        break;
                }

                $headers = array_merge($headers, json_decode($queueItem->getHeaders(), true));
                $headers['X-MAGENTO-UNIFIC-HMAC'] = $this->hmacHelper->generateHmac($queueData);
                $headers['X-MAGENTO-GUID'] = $queueItem->getGuid();

                $response = $this->restConnection->sendData($queueItem->getUrl(), $queueData, $headers, $type);

                if ($response->isSuccess()) {
                    $this->logger->createLog($queueItem, $response, 'Webhook sent succesfully and removed from queue');
                    $queueItem->delete();
                } else {
                    $queueItem->setRetryAmount($queueItem->getRetryAmount() + 1);
                    $queueItem->setResponseHttpCode($response->getStatusCode());
                    $queueItem->save();

                    if ($queueItem->getRetryAmount() == $queueItem->getMaxRetryAmount()) {
                        $this->logger->createLog($queueItem, $response, 'Webhook reached maximum retry amount. Sending is stopped until a retry is forced.');
                    } else {
                        $this->logger->createLog($queueItem, $response, 'Webhook failed, queueing retry attempt ' . $queueItem->getRetryAmount() . '/' . $queueItem->getMaxRetryAmount());
                    }
                }
            } else {
                $this->logger->createLogForCanceledWebhook($queueItem, 'Message not send and removed from queue, no valid endpoint configured');
                $queueItem->delete();
            }
        }
    }
}
